/*
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;		LCD-Routinen		;;
;;		============		;;
;;		(c)Simon			;;
;;	4-Bit-Interface			;;
;;	DB4-DB7:	PC0-PC3		;;
;;	RS:		PC4				;;
;;	E:		PC5				;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
*/
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "lcd.h"
#include "types.h"
#include "delay.h"


prog_uchar DISP_TEXT[TEXTCNT][LINESIZE] = {
"Mr.Midi 2 Bootloader    ",
"Writing page #          ",
"DATA ERROR: Stopped     ",
"Done! Toggle Power!     "
};


void lcd_enable(void);

// Sendet ein Datenbyte an das LCD
void lcd_data(u08 d) {
	LCD = (d>>4)|0x10;
	lcd_enable();
	LCD = (d&0x0f)|0x10;
	lcd_enable();
	delay_us(50);
}

// Sendet einen Befehl an das LCD
void lcd_command(u08 d) {
	LCD = (d>>4);
	lcd_enable();
	LCD = (d&0x0f);
	lcd_enable();
	delay_us(50);
}

// Erzeugt den Enable-Puls
void lcd_enable(void) {
#ifdef ADC_TEMPO
	LCD2 |= 1 << EN_PIN;
	delay_us(1);
	LCD2 &= ~(1 << EN_PIN);
#else
	LCD |= 1 << EN_PIN;
	delay_us(1);
	LCD &= ~(1 << EN_PIN);
#endif
}

// Sendet den Befehl zur Lschung des Displays
void lcd_clear(void) {
	lcd_command(LCD_CLEAR);
	delay_ms(5);
}

// Gibt einen String aus
void lcd_string(u08 idx, u08 row) {
	u08 i;
	lcd_setcur(0, row);
	for (i=0; i<LINESIZE; i++)
		lcd_data((u08)pgm_read_byte( &(DISP_TEXT[idx][i]) ));
}

// Setzt den Cursor
void lcd_setcur(u08 x, u08 y) {
	if (y==1)
		x += 64;
	lcd_command(0x80+x);
}

// Hexadezimale Ausgabe Nibble
void lcd_nibble(u08 d) {
	if (d > 9)
		lcd_data(d+'A'-10);
	else
		lcd_data(d+'0');
}

// Hexadezimale Ausgabe u08
void lcd_hex_u08(u08 d) {
	lcd_nibble(d>>4);
	lcd_nibble(d&0x0f);
}

// Hexadezimale Ausgabe u16
void lcd_hex_u16(u16 d) {
	lcd_hex_u08((u08)(d>>8));
	lcd_hex_u08((u08)d);
}
